@@ -13,18 +13,18 @@ import sys
 import types
 import weakref
 
-import pyutilib.th as unittest
+import pyomo.common.unittest as unittest
 
 from pyomo.common import DeveloperError
 from pyomo.common.deprecation import (
-    deprecated, deprecation_warning, relocated_module_attribute,
+    deprecated, deprecation_warning, relocated_module_attribute, RenamedClass
 )
 from pyomo.common.log import LoggingIntercept
 
-from six import StringIO
+from io import StringIO
 
 import logging
-logger = logging.getLogger('pyomo.common')
+logger = logging.getLogger('local')
 
 
 class TestDeprecated(unittest.TestCase):
@@ -32,7 +32,7 @@ class TestDeprecated(unittest.TestCase):
 
     def test_deprecation_warning(self):
         DEP_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
             deprecation_warning(None, version='1.2', remove_in='3.4')
 
         self.assertIn('DEPRECATED: This has been deprecated',
@@ -41,7 +41,7 @@ class TestDeprecated(unittest.TestCase):
                       DEP_OUT.getvalue().replace('\n',' '))
 
         DEP_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
             deprecation_warning("custom message here", version='1.2', remove_in='3.4')
 
         self.assertIn('DEPRECATED: custom message here',
@@ -51,11 +51,27 @@ class TestDeprecated(unittest.TestCase):
 
 
     def test_no_version_exception(self):
-        with self.assertRaises(DeveloperError):
+        with self.assertRaisesRegex(
+                DeveloperError, "@deprecated missing initial version"):
             @deprecated()
             def foo():
                 pass
 
+        with self.assertRaisesRegex(
+                DeveloperError, "@deprecated missing initial version"):
+            @deprecated()
+            class foo(object):
+                pass
+
+        # But no exception if the class can infer a version from the
+        # __init__ (or __new__ or __new_member__)
+        @deprecated()
+        class foo(object):
+            @deprecated(version="1.2")
+            def __init__(self):
+                pass
+        self.assertIn('.. deprecated:: 1.2', foo.__doc__)
+
     def test_no_doc_string(self):
         # Note: No docstring, else nose replaces the function name with
         # the docstring in output.
@@ -64,14 +80,15 @@ class TestDeprecated(unittest.TestCase):
         def foo(bar='yeah'):
             logger.warning(bar)
 
-        self.assertIn('DEPRECATION WARNING: This function has been deprecated',
-                      foo.__doc__)
+        self.assertIn(
+            '.. deprecated:: test\n   This function has been deprecated',
+            foo.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -83,8 +100,8 @@ class TestDeprecated(unittest.TestCase):
         # Test that the function argument gets passed in
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo("custom")
         # Test that the function produces output
         self.assertNotIn('yeah', FCN_OUT.getvalue())
@@ -105,15 +122,16 @@ class TestDeprecated(unittest.TestCase):
             """
             logger.warning(bar)
 
-        self.assertIn('DEPRECATION WARNING: This function has been deprecated',
-                      foo.__doc__)
+        self.assertIn(
+            '.. deprecated:: test\n   This function has been deprecated',
+            foo.__doc__)
         self.assertIn('I am a good person.', foo.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -125,8 +143,8 @@ class TestDeprecated(unittest.TestCase):
         # Test that the function argument gets passed in
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo("custom")
         # Test that the function produces output
         self.assertNotIn('yeah', FCN_OUT.getvalue())
@@ -147,15 +165,16 @@ class TestDeprecated(unittest.TestCase):
             """
             logger.warning(bar)
 
-        self.assertIn('DEPRECATION WARNING: This is a custom message',
-                      foo.__doc__)
+        self.assertIn(
+            '.. deprecated:: test\n   This is a custom message',
+            foo.__doc__)
         self.assertIn('I am a good person.', foo.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -167,8 +186,8 @@ class TestDeprecated(unittest.TestCase):
         # Test that the function argument gets passed in
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo("custom")
         # Test that the function produces output
         self.assertNotIn('yeah', FCN_OUT.getvalue())
@@ -178,8 +197,9 @@ class TestDeprecated(unittest.TestCase):
         self.assertIn('DEPRECATED: This is a custom message',
                       DEP_OUT.getvalue())
 
+
     def test_with_custom_logger(self):
-        @deprecated('This is a custom message', logger='pyomo.common',
+        @deprecated('This is a custom message', logger='local',
                     version='test')
         def foo(bar='yeah'):
             """Show that I am a good person.
@@ -189,15 +209,16 @@ class TestDeprecated(unittest.TestCase):
             """
             logger.warning(bar)
 
-        self.assertIn('DEPRECATION WARNING: This is a custom message',
-                      foo.__doc__)
+        self.assertIn(
+            '.. deprecated:: test\n   This is a custom message',
+            foo.__doc__)
         self.assertIn('I am a good person.', foo.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -210,8 +231,8 @@ class TestDeprecated(unittest.TestCase):
         # Test that the function argument gets passed in
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo("custom")
         # Test that the function produces output
         self.assertNotIn('yeah', FCN_OUT.getvalue())
@@ -221,20 +242,23 @@ class TestDeprecated(unittest.TestCase):
         # Test that the deprecation warning was logged
         self.assertNotIn('DEPRECATED:', DEP_OUT.getvalue())
 
+
     def test_with_class(self):
         @deprecated(version='test')
         class foo(object):
             def __init__(self):
                 logger.warning('yeah')
 
-        self.assertIn('DEPRECATION WARNING: This class has been deprecated',
-                      foo.__doc__)
+        self.assertIs(type(foo), type)
+        self.assertIn(
+            '.. deprecated:: test\n   This class has been deprecated',
+            foo.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -252,14 +276,15 @@ class TestDeprecated(unittest.TestCase):
             def bar(self):
                 logger.warning('yeah')
 
-        self.assertIn('DEPRECATION WARNING: This function has been deprecated',
-                      foo.bar.__doc__)
+        self.assertIn(
+            '.. deprecated:: test\n   This function has been deprecated',
+            foo.bar.__doc__)
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo().bar()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -276,16 +301,17 @@ class TestDeprecated(unittest.TestCase):
             def bar(self):
                 logger.warning('yeah')
 
-        self.assertIn('DEPRECATION WARNING: This function has been deprecated',
-                      foo.bar.__doc__)
-        self.assertIn('(deprecated in 1.2, will be removed in 3.4)',
+        self.assertIn(
+            '.. deprecated:: 1.2\n   This function has been deprecated',
+            foo.bar.__doc__)
+        self.assertIn('(will be removed in 3.4)',
                       foo.bar.__doc__.replace('\n',' '))
 
         # Test the default argument
         DEP_OUT = StringIO()
         FCN_OUT = StringIO()
-        with LoggingIntercept(DEP_OUT, 'pyomo.core'):
-            with LoggingIntercept(FCN_OUT, 'pyomo.common'):
+        with LoggingIntercept(DEP_OUT, 'pyomo'):
+            with LoggingIntercept(FCN_OUT, 'local'):
                 foo().bar()
         # Test that the function produces output
         self.assertIn('yeah', FCN_OUT.getvalue())
@@ -311,7 +337,7 @@ class TestRelocated(unittest.TestCase):
         warning = "DEPRECATED: the 'myFoo' class has been moved to " \
                   "'pyomo.common.tests.relocated.Bar'"
         OUT = StringIO()
-        with LoggingIntercept(OUT, 'pyomo.core'):
+        with LoggingIntercept(OUT, 'pyomo'):
             from pyomo.common.tests.test_deprecated import myFoo
         self.assertEqual(myFoo.data, 42)
         self.assertIn(warning, OUT.getvalue().replace('\n', ' '))
@@ -330,7 +356,7 @@ class TestRelocated(unittest.TestCase):
                   "'pyomo.common.tests.relocated.Bar'"
 
         OUT = StringIO()
-        with LoggingIntercept(OUT, 'pyomo.core'):
+        with LoggingIntercept(OUT, 'pyomo'):
             self.assertIs(relocated.Foo_2, relocated.Bar)
             self.assertEqual(relocated.Foo_2.data, 42)
         self.assertIn(warning, OUT.getvalue().replace('\n', ' '))
@@ -343,7 +369,7 @@ class TestRelocated(unittest.TestCase):
                   "'pyomo.common.tests.test_deprecated.Bar'"
 
         OUT = StringIO()
-        with LoggingIntercept(OUT, 'pyomo.core'):
+        with LoggingIntercept(OUT, 'pyomo'):
             from pyomo.common.tests.relocated import Foo
             self.assertEqual(Foo.data, 21)
         self.assertIn(warning, OUT.getvalue().replace('\n', ' '))
@@ -358,5 +384,152 @@ class TestRelocated(unittest.TestCase):
                 "(?:'module' object) has no attribute 'Baz'"):
             relocated.Baz.data
 
+
+class TestRenamedClass(unittest.TestCase):
+    def test_renamed(self):
+        class NewClass(object):
+            attr = 'NewClass'
+
+        class NewClassSubclass(NewClass):
+            pass
+
+        # The deprecated class does not generate a warning
+        out = StringIO()
+        with LoggingIntercept(out):
+            class DeprecatedClass(metaclass=RenamedClass):
+                __renamed__new_class__ = NewClass
+                __renamed__version__ = 'X.y'
+        self.assertEqual(out.getvalue(), "")
+
+        # Inheriting from the deprecated class generates the warning
+        out = StringIO()
+        with LoggingIntercept(out):
+            class DeprecatedClassSubclass(DeprecatedClass):
+                attr = 'DeprecatedClassSubclass'
+        self.assertRegex(
+            out.getvalue().replace("\n", " ").strip(),
+            r"^DEPRECATED: Declaring class 'DeprecatedClassSubclass' "
+            r"derived from 'DeprecatedClass'.  "
+            r"The class 'DeprecatedClass' has been renamed to 'NewClass'  "
+            r"\(deprecated in X.y\) \(called from [^\)]*\)$",
+        )
+
+        # Inheriting from a class derived from the deprecated class does
+        # not generate a warning
+        out = StringIO()
+        with LoggingIntercept(out):
+            class DeprecatedClassSubSubclass(DeprecatedClassSubclass):
+                attr = 'DeprecatedClassSubSubclass'
+        self.assertEqual(out.getvalue(), "")
+
+        #
+        # Test class creation
+        #
+
+        out = StringIO()
+        with LoggingIntercept(out):
+            newclass = NewClass()
+            newclasssubclass = NewClassSubclass()
+        self.assertEqual(out.getvalue(), "")
+
+        out = StringIO()
+        with LoggingIntercept(out):
+            deprecatedclass = DeprecatedClass()
+        self.assertRegex(
+            out.getvalue().replace("\n", " ").strip(),
+            r"^DEPRECATED: Instantiating class 'DeprecatedClass'.  "
+            r"The class 'DeprecatedClass' has been renamed to 'NewClass'  "
+            r"\(deprecated in X.y\) \(called from [^\)]*\)$",
+        )
+
+        # Instantiating a class derived from the deprecaed class does
+        # not generate a warning (the warning is generated when the
+        # class is declared)
+        out = StringIO()
+        with LoggingIntercept(out):
+            deprecatedsubclass = DeprecatedClassSubclass()
+            deprecatedsubsubclass = DeprecatedClassSubSubclass()
+        self.assertEqual(out.getvalue(), "")
+
+        #
+        # Test isinstance
+        #
+        out = StringIO()
+        with LoggingIntercept(out):
+            self.assertIsInstance(deprecatedsubclass, NewClass)
+            self.assertIsInstance(deprecatedsubsubclass, NewClass)
+        self.assertEqual(out.getvalue(), "")
+
+        for obj in (newclass, newclasssubclass, deprecatedclass,
+                    deprecatedsubclass, deprecatedsubsubclass):
+            out = StringIO()
+            with LoggingIntercept(out):
+                self.assertIsInstance(obj, DeprecatedClass)
+            self.assertRegex(
+                out.getvalue().replace("\n", " ").strip(),
+                r"^DEPRECATED: Checking type relative to 'DeprecatedClass'.  "
+                r"The class 'DeprecatedClass' has been renamed to 'NewClass'  "
+                r"\(deprecated in X.y\) \(called from [^\)]*\)$",
+            )
+
+        #
+        # Test issubclass
+        #
+
+        out = StringIO()
+        with LoggingIntercept(out):
+            self.assertTrue(issubclass(DeprecatedClass, NewClass))
+            self.assertTrue(issubclass(DeprecatedClassSubclass, NewClass))
+            self.assertTrue(issubclass(DeprecatedClassSubSubclass, NewClass))
+        self.assertEqual(out.getvalue(), "")
+
+        for cls in (NewClass, NewClassSubclass, DeprecatedClass,
+                    DeprecatedClassSubclass, DeprecatedClassSubSubclass):
+            out = StringIO()
+            with LoggingIntercept(out):
+                self.assertTrue(issubclass(cls, DeprecatedClass))
+            self.assertRegex(
+                out.getvalue().replace("\n", " ").strip(),
+                r"^DEPRECATED: Checking type relative to 'DeprecatedClass'.  "
+                r"The class 'DeprecatedClass' has been renamed to 'NewClass'  "
+                r"\(deprecated in X.y\) \(called from [^\)]*\)$",
+            )
+
+        #
+        # Test class attributes
+        #
+        self.assertEqual(newclass.attr, 'NewClass')
+        self.assertEqual(newclasssubclass.attr, 'NewClass')
+        self.assertEqual(deprecatedclass.attr, 'NewClass')
+        self.assertEqual(deprecatedsubclass.attr,
+                         'DeprecatedClassSubclass')
+        self.assertEqual(deprecatedsubsubclass.attr,
+                         'DeprecatedClassSubSubclass')
+        self.assertEqual(NewClass.attr, 'NewClass')
+        self.assertEqual(NewClassSubclass.attr, 'NewClass')
+        self.assertEqual(DeprecatedClass.attr, 'NewClass')
+        self.assertEqual(DeprecatedClassSubclass.attr,
+                         'DeprecatedClassSubclass')
+        self.assertEqual(DeprecatedClassSubSubclass.attr,
+                         'DeprecatedClassSubSubclass')
+
+    def test_renamed_errors(self):
+        class NewClass(object):
+            pass
+
+        with self.assertRaisesRegex(
+                TypeError, "Declaring class 'DeprecatedClass' using the "
+                "RenamedClass metaclass, but without specifying the "
+                "__renamed__new_class__ class attribute"):
+            class DeprecatedClass(metaclass=RenamedClass):
+                __renamed_new_class__ = NewClass
+
+        with self.assertRaisesRegex(
+                TypeError, "Declaring class 'DeprecatedClass' using the "
+                "RenamedClass metaclass, but without specifying the "
+                "__renamed__version__ class attribute"):
+            class DeprecatedClass(metaclass=RenamedClass):
+                __renamed__new_class__ = NewClass
+
 if __name__ == '__main__':
     unittest.main()
